package com.tzld.commons.llm.azure.gpt;

import com.alibaba.fastjson.JSONObject;
import com.tzld.commons.llm.common.Constant;
import com.tzld.commons.llm.common.ConversationManager;
import com.tzld.commons.llm.gpt.param.OpenAiGptParam;
import com.tzld.commons.llm.gpt.util.AigcUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.time.StopWatch;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.http.HttpEntity;
import org.springframework.http.HttpMethod;
import org.springframework.stereotype.Component;
import org.springframework.web.client.RestTemplate;

import javax.annotation.Resource;
import java.util.Objects;

/**
 * @author sunxy
 */
@Slf4j
@Component(value = Constant.AZURE_GPT_CONVERSATION_MANAGER)
public class AzureGptManager implements ConversationManager {

    @Resource
    private RestTemplate restTemplate;
    @Value("${azure.openai.conversation.url:}")
    private String conversationUrl;

    @Override
    public JSONObject conversation(OpenAiGptParam openAiGptParam, String auth) {
        StopWatch st = StopWatch.createStarted();
        if (StringUtils.isBlank(auth)) {
            return null;
        }
        openAiGptParam.setModel(null);
        String requestParams = JSONObject.toJSONString(openAiGptParam);
        HttpEntity<String> entity = new HttpEntity<>(requestParams,
                AigcUtils.getAigcHeaders(auth));
        JSONObject result = doRequestAzureOpenai(requestParams, entity, openAiGptParam.getModel());
        log.info("request auth {} param {} result {}  cost {}", auth, JSONObject.toJSONString(openAiGptParam),
                Objects.isNull(result) ? null : result.toJSONString(), st.getTime());
        return result;
    }

    private JSONObject doRequestAzureOpenai(String requestParams, HttpEntity<String> entity, String model) {
        log.info("azure openai chat request param {}", requestParams);
        JSONObject jsonObject = null;
        String requestUrl = String.format(conversationUrl, model);
        for (int i = 0; i < 3; i++) {
            try {
                jsonObject = restTemplate.exchange(requestUrl, HttpMethod.POST,
                        entity, JSONObject.class).getBody();
                if (jsonObject == null || Objects.nonNull(jsonObject.get("choices"))) {
                    break;
                }
            } catch (Exception e) {
                log.error("request openAi error ", e);
            }
        }
        return jsonObject;
    }

    @Override
    public String chooseAuth() {
        return null;
    }
}
