package com.tzld.commons.llm.common.call;

import com.alibaba.fastjson.JSONObject;
import com.tzld.commons.llm.common.Constant;
import com.tzld.commons.llm.common.ConversationManager;
import com.tzld.commons.llm.common.enums.LLMPlatformTypeEnum;
import com.tzld.commons.llm.gpt.param.OpenAiGptParam;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Component;

import java.util.Arrays;
import java.util.Map;
import java.util.Optional;
import java.util.concurrent.ThreadLocalRandom;

/**
 * @author sunxy
 */
@Component
public class OpenaiCaller {

    @Autowired
    private Map<String, ConversationManager> conversationManagerMap;


    public JSONObject call(OpenAiGptParam param) {
        // 随机排序, 防止每次都是同一个平台, 取第一个元素
        Optional<LLMPlatformTypeEnum> platformTypeEnum = Arrays.stream(LLMPlatformTypeEnum.values())
                .min((o1, o2) -> ThreadLocalRandom.current().nextInt(-1, 2));
        return platformTypeEnum.map(llmPlatformTypeEnum -> {
            try {
                return call(param, llmPlatformTypeEnum);
            } catch (Exception e) {
                throw new RuntimeException(e);
            }
        }).orElse(null);
    }

    public JSONObject call(OpenAiGptParam param, LLMPlatformTypeEnum platform) throws IllegalAccessException {
        if (platform == null) {
            throw new IllegalAccessException("platform can't be null");
        }
        ConversationManager conversationManager = null;
        switch (platform) {
            case OPENAI_OFFICIAL:
                conversationManager = conversationManagerMap.get(Constant.OFFICIAL_GPT_CONVERSATION_MANAGER);
                break;
            case AZURE_OPENAI:
                conversationManager = conversationManagerMap.get(Constant.AZURE_GPT_CONVERSATION_MANAGER);
                break;
            default:
                break;
        }
        if (conversationManager == null) {
            return null;
        }
        return conversationManager.conversation(param, conversationManager.chooseAuth());
    }

}
